@@ -28,7 +28,11 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+
 #include "gtkhtml.h"
+#include "htmlengine.h"
+#include "htmlundo.h"
+
 #include "persist-file.h"
 #include "editor-control-factory.h"
 
@@ -36,6 +40,8 @@ static BonoboObjectClass *gtk_html_persist_file_parent_class;
 
 static void impl_save (PortableServer_Servant servant, const CORBA_char *path, CORBA_Environment * ev);
 static void impl_load (PortableServer_Servant servant, const CORBA_char *path, CORBA_Environment * ev);
+static CORBA_boolean impl_isDirty (PortableServer_Servant servant, CORBA_Environment *ev);
+static CORBA_char *  impl_getCurrentFile (PortableServer_Servant servant, CORBA_Environment *ev);
 
 static void
 finalize (GObject *object)
@@ -47,6 +53,11 @@ finalize (GObject *object)
 		file->html = NULL;
 	}
 
+	if (file->uri) {
+		g_free (file->uri);
+		file->uri = NULL;
+	}
+
 	G_OBJECT_CLASS (gtk_html_persist_file_parent_class)->finalize (object);
 }
 
@@ -67,9 +78,11 @@ gtk_html_persist_file_class_init (GtkHTMLPersistFileClass *klass)
 
 	epv->load = impl_load;
 	epv->save = impl_save;
+	epv->getCurrentFile = impl_getCurrentFile;
 
 	object_class->finalize = finalize;
 	persist_class->get_content_types = get_content_types;
+	persist_class->epv.isDirty = impl_isDirty;
 }
 
 GType
@@ -110,6 +123,8 @@ gtk_html_persist_file_new (GtkHTML *html)
 
 	g_object_ref (html);
 	GTK_HTML_PERSIST_FILE (file)->html = html;
+	GTK_HTML_PERSIST_FILE (file)->uri = NULL;
+	GTK_HTML_PERSIST_FILE (file)->saved_step_count = -1;
 
 	return file;
 }
@@ -160,9 +175,16 @@ impl_load (PortableServer_Servant servant, const CORBA_char *path, CORBA_Environ
 		if (was_editable)
 			gtk_html_set_editable (file->html, TRUE);
 	}
+
+	/* Free old uri. */
+	if (file->uri)
+		g_free (file->uri);
+
+	/* Save the file's uri. */
+	file->uri = g_strdup (path);
 }
 
-static gboolean
+static CORBA_boolean
 save_receiver  (const HTMLEngine *engine, const char *data, unsigned int len, void *user_data)
 {
 	int fd;
@@ -189,12 +211,42 @@ impl_save (PortableServer_Servant servant, const CORBA_char *path, CORBA_Environ
 	GtkHTMLPersistFile *file = GTK_HTML_PERSIST_FILE (bonobo_object_from_servant (servant));
 	int fd;
 
-	fd = open (path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+	fd = open (path, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 
 	if (fd == -1)
 		return;
 
 	gtk_html_save (file->html, (GtkHTMLSaveReceiverFn) save_receiver, GINT_TO_POINTER (fd));
-
 	close (fd);
+	file->html->engine->saved_step_count = html_undo_get_step_count (file->html->engine->undo);
+
+	/* Free old uri. */
+	if (file->uri)
+		g_free (file->uri);
+
+	/* Save the file's uri. */
+	file->uri = g_strdup (path);
+}
+
+static CORBA_boolean
+impl_isDirty (PortableServer_Servant servant, CORBA_Environment *ev)
+{
+	GtkHTMLPersistFile *file = GTK_HTML_PERSIST_FILE (bonobo_object_from_servant (servant));
+
+	/* I don't think we drop Undos on Save-ing. */
+	return file->saved_step_count != -1 && file->html->engine->saved_step_count == html_undo_get_step_count (file->html->engine->undo) ? CORBA_FALSE : CORBA_TRUE;
+}
+
+static CORBA_char *
+impl_getCurrentFile (PortableServer_Servant servant, CORBA_Environment *ev)
+{
+	GtkHTMLPersistFile *file = GTK_HTML_PERSIST_FILE (bonobo_object_from_servant (servant));
+
+	/* Raise NoCurrentName exception. */
+	if (!file->uri) {
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION, ex_Bonobo_PersistFile_NoCurrentName, NULL);
+		return NULL;
+	}
+
+	return CORBA_string_dup (file->uri);
 }
